/*
/*
*
*    OPEN-XCHANGE legal information
*
*    All intellectual property rights in the Software are protected by
*    international copyright laws.
*
*
*    In some countries OX, OX Open-Xchange, open xchange and OXtender
*    as well as the corresponding Logos OX Open-Xchange and OX are registered
*    trademarks.
*    The use of the Logos is not covered by the GNU General Public License.
*    Instead, you are allowed to use these Logos according to the terms and
*    conditions of the Creative Commons License, Version 2.5, Attribution,
*    Non-commercial, ShareAlike, and the interpretation of the term
*    Non-commercial applicable to the aforementioned license is published
*    on the web site http://www.open-xchange.com/EN/legal/index.html.
*
*    Please make sure that third-party modules and libraries are used
*    according to their respective licenses.
*
*    Any modifications to this package must retain all copyright notices
*    of the original copyright holder(s) for the original code used.
*
*    After any such modifications, the original and derivative code shall remain
*    under the copyright of the copyright holder(s) and/or original author(s)per
*    the Attribution and Assignment Agreement that can be located at
*    http://www.open-xchange.com/EN/developer/. The contributing author shall be
*    given Attribution for the derivative code and a license granting use.
*
*     Copyright (C) 2016 OX Software GmbH
*     Mail: info@open-xchange.com
*
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License, Version 2 as published
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc., 59
*     Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*/

package com.openexchange.office.document;

import java.lang.ref.WeakReference;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.springframework.stereotype.Service;

import com.openexchange.office.document.api.Document;
import com.openexchange.office.document.api.DocumentDisposer;
import com.openexchange.office.tools.annotation.RegisteredService;

@Service
@RegisteredService(registeredClass=DocumentDisposer.class)
public class DocumentDisposerImpl implements DocumentDisposer  {

	private ConcurrentMap<String, WeakReference<Document>> aDocumentsMap = new ConcurrentHashMap<>();

    //-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see com.openexchange.office.document.DocumentDisposer#getDocument(java.lang.String)
	 */
	@Override
	public Document getDocument(String docUID) {
		WeakReference<Document> aWeakRef = aDocumentsMap.get(docUID);

		if (null != aWeakRef)
			return aWeakRef.get();

		return null;
	}

    //-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see com.openexchange.office.document.DocumentDisposer#addDocumentRef(com.openexchange.office.document.api.Document)
	 */
	@Override
	public void addDocumentRef(Document aDoc) {
		WeakReference<Document> aWeakRef = new WeakReference<>(aDoc);
		aDocumentsMap.putIfAbsent(aDoc.getDocUID().getValue(), aWeakRef);
	}

    //-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see com.openexchange.office.document.DocumentDisposer#removeDocumentRef(java.lang.String)
	 */
	@Override
	public void removeDocumentRef(String docUID) {
		aDocumentsMap.remove(docUID);
	}
}
