/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.rest;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.openexchange.ajax.requesthandler.AJAXActionService;
import com.openexchange.ajax.requesthandler.AJAXActionServiceFactory;
import com.openexchange.office.document.api.TemplateFilesScannerFactory;
import com.openexchange.office.tools.common.osgi.context.OsgiBundleContextAndActivator;
import com.openexchange.office.tools.common.osgi.context.OsgiBundleContextAware;

/**
 * {@link DocumentRESTActionFactory}
 *
 * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
 */
@Service
public class DocumentRESTActionFactory implements AJAXActionServiceFactory, InitializingBean, OsgiBundleContextAware {

	private static final String AJAX_MODULE_ID = "oxodocumentfilter";
	
    private final Map<String, AJAXActionService> actions = new HashMap<>();

    @Autowired
    private TemplateFilesScannerFactory templateFilesScannerFactory;
    
    @Override
	public void afterPropertiesSet() throws Exception {
    	templateFilesScannerFactory.create(TemplateFilesScannerFactory.TemplateFilterType.ALL_DOCUMENTS);    	
	}

	@Override
    public AJAXActionService createActionService(final String action) {
        return (actions.get(action));
    }

    public Collection<? extends AJAXActionService> getSupportedServices() {
        return java.util.Collections.unmodifiableCollection(actions.values());
    }

	@Override
	public void setApplicationContext(OsgiBundleContextAndActivator bundleCtx) {
		Collection<DocumentRESTAction> documentRESTActions = bundleCtx.getServicesOfType(DocumentRESTAction.class);
		documentRESTActions.forEach(act -> {
			actions.put(act.getClass().getSimpleName().replace("Action", "").toLowerCase(), act);
		});
		bundleCtx.registerAjaxActionServiceFactoryToOSGI(this, AJAX_MODULE_ID,  true);
	}
}
